#include "inkey.ch"
#include "button.ch"
#include "setcurs.ch"
#include "set.ch"

#xtranslate CRLF => chr(13)+chr(10)

/*
        values for generation, saved in cargo (array) of item:
*/
#define CHECKED         1       // cVal
#define DATA            2       // cVal
#define ENABLED         3       // cVal
#define SHORTCUT        4       // nVal
#define SEPARATOR       5       // lVal


Procedure GenMENU(cDefIn, cDefOut)

/*
        cDefIn          DBF to restore-from-def-file
        cDefOut         DBF to save-to-def-file
*/

// local oTB
local lCyklus:=.T.
cColorDat:="W+/B,N/BG"
cColorSet:="N/BG,BG+/B,R/BG,R+/B,N+/W,N/BG"
aNulCargo:={"", "", "", 0, .F.}

if MPresent()
  SET EVENTMASK to INKEY_ALL
  MSetCursor(.T.)
endif

oTB:=ReadDef(cDefIn)                    // read saved def-menu (if exist and asked)
if oTB==NIL
  oTB:=CreateBase()                     // create base of menu
endif

DO WHILE lCyklus
  SetFKeys(@oTB)                        // set function keys
  lCyklus:=RunMenu(@oTB)                // show&edit menu
ENDDO

WriteDef(cDefOut, @oTB)                 // save defmenu, if asked
return .T.

/* ----------------------------------------------------------- */

static FUNCTION CreateBase()    // create base for menu
local oTB:=TopBar(0, 0, MaxCol())
local oP:=PopUp()
local oI:=MenuItem("&Item 01-01", {||.T.})

oTB:cargo:=Aclone(aNulCargo)
oTB:colorSpec:=cColorSet
oP:colorSpec:=cColorSet
oI:cargo:=aClone(aNulCargo)
oP:AddItem(oI)
oI:=MenuItem("&COLUMN 01", oP)
oI:cargo:=aClone(aNulCargo)
oTB:AddItem(oI)
return oTB

/* ----------------------------------------------------------- */

static PROCEDURE SetFkeys(oTB)          // set function keys for editing

SetKey(K_INS, {|| InsItem(oTB, 0) } )           // INS insert before current
SetKey(K_CTRL_INS, {|| InsItem(oTB, 0) } )      // Ctrl-INS insert before current
SetKey(K_ALT_INS, {|| InsItem(oTB, 1) } )       // Alt-INS insert item next to current

SetKey(K_DEL, {|| DelItem(oTB) } )              // DEL delete current

SetKey(K_CTRL_ENTER, {|| ChangeItem(oTB) } )    // Ctrl-ENTER edit current item

SetKey(K_F2, {|| SetPColor(oTB) } )             // set colorset of TopBar or PopUp
SetKey(K_F3, {|| SetTopBar(oTB) } )             // set attributes of TopBar
SetKey(K_F7, {|| SaveDef(oTB) } )               // save def to output-def-file
SetKey(K_F10, {|| RedrawScr(oTB) } )            // redraw screen

SetKey(K_F5, {|| GenPrg(oTB)})                  // generate .PRG

return .T.


/* ----------------------------------------------------------- */

static FUNCTION RunMenu(oTB)            // show and edit menu
CLS
ShowHelp()
MenuModal(oTB, 1, MaxRow(), 0, MaxCol())

return .not.(Alert("Exit program", {"No ", "Yes"}) = 2)
/* ----------------------------------------------------------- */

static PROCEDURE ReadDef(cDin)          // read input def-file, if exist
local oTB:=TopBar(0, 0, MaxCol())
if cDin = NIL                           // input def-file not set
  return NIL
endif
cDin:=iif(at(".", cDin)=0, cDin+".DBF", cDin )
if .not.file(cDin)                      // not found
  Alert("Input def-file not found:;"+cDin)
  return NIL
endif
use (cDin) alias RD             // 1-st record - TopBar attributes
oTB:colorSpec   :=Alltrim(RD->SCOLOR)
oTB:left        :=val(RD->SDATA)
oTB:right       :=val(RD->SCHECKED)
oTB:row         :=val(RD->SENABLED)
oTB:cargo       :=Aclone(aNulCargo)
skip
RestLevel(@oTB, 0)
close RD
return oTB

static PROCEDURE RestLevel(oPop, nLevel)
local oP, oI
local nL:=nLevel+1

select RD
Do while .not.Eof()
  if RD->SLEVEL < nL                    // back to higher level
     return .T.
  endif
  Do case
    case RD->STYP == "P"                // knote - go down
      oP:=PopUp()
      oP:colorSpec:=Alltrim(RD->SCOLOR)
      oP:cargo:=Aclone(aNulCargo)
      oI:=MenuItem(Alltrim(RD->SCAPTION), oP)
      oI:cargo:=Aclone(aNulCargo)
      oI:message:=Alltrim(RD->SMESSAGE)
      oI:cargo[CHECKED]:=Alltrim(RD->SCHECKED)
      oPop:AddItem(oI)
      skip
      RestLevel(@oP, nL)

    case RD->STYP == "S"                // same level, separator
      oI:=MenuItem(Alltrim(RD->SCAPTION), {||.T.})
      oI:cargo:=Aclone(aNulCargo)
      oI:cargo[SEPARATOR]:=.T.
      oPop:AddItem(oI)
      skip

    case RD->STYP == "I"                // same level, menu item
      oI:=MenuItem(Alltrim(RD->SCAPTION), {||.T.})
      oI:cargo:=Aclone(aNulCargo)
      oI:cargo[DATA]:=Alltrim(RD->SDATA)
      oI:cargo[ENABLED]:=Alltrim(RD->SENABLED)
      oI:cargo[CHECKED]:=Alltrim(RD->SCHECKED)
      oI:cargo[SHORTCUT]:=RD->SSHORTCUT
      oI:cargo[SEPARATOR]:=RD->SSEPARATOR
      oI:message:=Alltrim(RD->SMESSAGE)
      oI:style:=RD->SSTYLE

      oPop:AddItem(oI)
      skip

  endcase
EndDo
return .T.
/* ----------------------------------------------------------- */

static PROCEDURE SaveDef(oTB)            // save def to DBF-file
local nHandle, cMemo
local cFile:="NEW_MENU.DBF"+space(44)
local aSaveFkey:=f_Fkeys("A")           // save and clear fkeys
local aSet:=f_Set("A")

SetColor(cColorDat)

@ 10, 10 clear to 14, 69
@ 10, 10       to 14, 69
@ 10, 25 say "< Name of saved defmenu: >"
@ 12, 12 get cFile
if f_Read()
  cFile:=Alltrim(cFile)
  WriteDef(cFile, oTB)
endif
f_Fkeys("R", aSaveFkey)
f_Set("R", aSet)
return .T.


/* ----------------------------------------------------------- */

static FUNCTION WriteDef(cDout, oTB)         // save def to def-file
local aStru:={                                          ;
                { "SLEVEL",     "N",   5, 0},           ;
                { "STYP",       "C",   1, 0},           ;
                { "SCAPTION",   "C",  80, 0},           ;
                { "SMESSAGE",   "C",  80, 0},           ;
                { "SDATA",      "C",  80, 0},           ;
                { "SCHECKED",   "C",  80, 0},           ;
                { "SENABLED",   "C",  80, 0},           ;
                { "SCOLOR",     "C",  80, 0},           ;
                { "SSTYLE",     "C",   2, 0},           ;
                { "SSEPARATOR", "L",   1, 0},           ;
                { "SSHORTCUT",  "N",   5, 0}            ;
                                              }

if cDout=NIL
  return .F.
endif
if Alert("Save defmenu to: "+cDout)<>0
  DBcreate(cDout, aStru)
  use (cDout) alias SD

  append blank                  // 1-st record - TopBar attributes
  SD->SLEVEL    :=0
  SD->STYP      :="T"
  SD->SCOLOR    :=oTB:colorSpec
  SD->SDATA     :=str(oTB:left)
  SD->SCHECKED  :=str(oTB:right)
  SD->SENABLED  :=str(oTB:row)

  SaveLevel(@oTB, 0)
  close SD
endif
return .T.

/* ----------------------------------------------------------- */

static FUNCTION SaveLevel(oP, nLevel)
local nI, oI, oPP
local nL:=nLevel+1

For nI:=1 to oP:itemCount
  oI:=oP:GetItem(nI)
  select SD
  append blank
  SD->STYP     := "I"
  SD->SLEVEL   := nL
  SD->SCAPTION := oI:caption
  SD->SMESSAGE := oI:message
  SD->SDATA    := oI:cargo[DATA]
  SD->SENABLED := oI:cargo[ENABLED]
  SD->SCHECKED := oI:cargo[CHECKED]
  SD->SSHORTCUT:= oI:cargo[SHORTCUT]
  SD->SSEPARATOR:=oI:cargo[SEPARATOR]
  SD->SSTYLE   := oI:style

  do case
    case oI:isPopUp()
      oPP:=oI:data
      SD->STYP     := "P"
      SD->SCOLOR   := oPP:colorSpec
      SaveLevel(oPP, nL)
    case oI:cargo[SEPARATOR]
      SD->STYP     := "S"
  endcase
next
return .T.



/* ----------------------------------------------------------- */
/*
        returns active PopUp.
        NIL = active is only item of TopBar
*/

static FUNCTION GetPopUp(oTB)           // Get Active PopUp
local oIT:=oTB:GetItem(oTB:current)     // current TopBarItem (allways present)
local oP:=oPret:=NIL
local oIP

if oIT:IsPopUp()                        // is current TopBarItem of PopUp type?
  oP:=oIT:data
  Do while oP:current <> 0
    oIP:=oP:GetItem(oP:current)             // selected Item of PopUp
    oPret:=oP
    if oIP:IsPopUp()
      oP:=oIP:data
    else
      exit
    endif
  enddo
endif
return oPret

/* ----------------------------------------------------------- */

static FUNCTION InsItem(oTB, nPos)            // insert new item current+nPos
local oP:=GetPopUp(@oTB)                // get active PopUp
local oI:=EditItem()
if oI==NIL
  return .T.
endif
nPos:=iif(nPos=NIL, 0, nPos)
oP:=iif(oP==NIL, oTB, oP)

nPos:=Max(oP:current+nPos, 1)
if nPos > oP:GetLast()
  oP:AddItem(oI)
else
  oP:InsItem(nPos, oI)
endif
RedrawScr(oTB)

return .T.

/* ----------------------------------------------------------- */

static FUNCTION DelItem(oTB)            // delete current item
local oP:=GetPopUp(@oTB)                // get active PopUp
local nPos
oP:=iif(oP==NIL, oTB, oP)
nPos:=oP:current

if oP:itemCount<=1
  Alert("PopUp block must contain at least one Item")
  return .T.
endif
if oP:Current=oP:GetLast()
  oP:select(max(oP:GetPrev(), 1))
endif
oP:DelItem(nPos)
RedrawScr(oTB)
return .T.

/* ----------------------------------------------------------- */


static FUNCTION SetPColor(oTB)            // set color set of TopBar/PopUp
local aSet:=f_Set("A")
local cC, aC, nI
local oP:=GetPopUp(oTB)                 // get active PopUp
local aSaveFkey:=f_Fkeys("A")           // save and clear fkeys

SetKey(K_F1, {||SetColorH()})

if oP==NIL
  cC:=oTB:colorSpec
else
  cC:=oP:colorSpec
endif
aC:=f_Split(cC, ",")
For nI:=1 to len(aC)
  aC[nI, 1]:=PadR(aC[nI, 1], 8)
next
@ 0, 0 clear to MaxRow(), MaxCol()
@ 0, 0       to MaxRow(), MaxCol()
@ 0, 30 say "<Set colors of "+iif(oP==NIL, "TopBar>", "active PopUp>")
@ row()+2, 1 say "colors for UNSELECTED items:"
@ row()  , 50 get aC[1, 1]
@ row()  , 60 say "<F1>: help"
@ row()+1, 1 say "colors for SELECTED items:"
@ row()  , 50 get aC[2, 1]
@ row()+1, 1 say "colors for UNSELECTED ACCELERATOR KEY:"
@ row()  , 50 get aC[3, 1]
@ row()+1, 1 say "colors for SELECTED ACCELERATOR KEY:"
@ row()  , 50 get aC[4, 1]
@ row()+1, 1 say "colors for DISABLED items:"
@ row()  , 50 get aC[5, 1]
@ row()+1, 1 say "colors for BORDER:"
@ row()  , 50 get aC[6, 1]
if f_Read()
  cC:=Rtrim(aC[1,1]-","-aC[2,1]-","-aC[3,1]-","-aC[4,1]-","-aC[5,1]-","-aC[6,1])
  if oP==NIL
    oTB:colorSpec:=cC
  else
    oP:colorSpec:=cC
  endif
endif
f_Fkeys("R", aSaveFkey)
f_Set("R", aSet)
return .T.

static FUNCTION SetColorH()
local oG:=GetActive()
keyboard f_barva(oG:buffer, row()+1, 0, .T.)
return .T.

/* ----------------------------------------------------------- */


static FUNCTION SetTopBar(oTB)            // set attributes of TopBar
local aSet:=f_Set("A")
local aSaveFkey:=f_Fkeys("A")           // save and clear fkeys
local nRow:=oTB:row
local nLeft:=oTB:left
local nRight:=oTB:right

SetColor(cColorDat)

@ 0, 0 clear to MaxRow(), MaxCol()
@ 0, 0       to MaxRow(), MaxCol()
@ 0, 30 say "<Set attributes of TopBar >"
@ row()+2, 1 say "The row of TopBar menu:"
@ row()  , 50 get nRow picture "999" valid (nRow >=0 .and. nRow <=MaxRow())
@ row()+1, 1 say "The max. left column of TopBar menu:"
@ row()  , 50 get nLeft picture "999" valid (nLeft >=0 .and. nLeft <=MaxCol())
@ row()+1, 1 say "The max. right column of TopBar menu:"
@ row()  , 50 get nRight picture "999" valid (nRight >=nLeft .and. nRight <=MaxCol())
if f_Read()
    oTB:row:=nRow
    oTB:left:=nLeft
    oTB:right:=nRight
endif
f_Fkeys("R", aSaveFkey)
f_Set("R", aSet)
RedrawScr(oTB)
return .T.

/* ----------------------------------------------------------- */

static FUNCTION ShowHelp()
DevPos(MaxRow()-1, 0)
@ Row()-1,  0 say "F10:     redraw the screen and menu (useful in TopBar after insert/delete)"
@ Row()-2,  0 say "F7:      save current definition of menu to DBF-def-file"
@ Row()-1,  0 say "F5:      generate and edit the source PRG"
@ Row()-2,  0 say "F3:      set the attributes of TopBar"
@ Row()-1,  0 say "F2:      set the colorset of the current PopUp/TopBar"
@ Row()-2,  0 say "CtrlEnter: change the current item"
@ Row()-2,  0 say "Del:    delete current item"
@ Row()-2,  0 say "AltIns: insert new item next to the current"
@ Row()-1,  0 say "CtrlIns:insert new item before current"
@ Row()-1,  0 say "Ins:    insert new item before current"
@ Row()-3,  0 say "usage: GENMENU [ input-def-file [ output-def-file ] ]"
return .T.


/* ----------------------------------------------------------- */


static FUNCTION EditItem(oMI)            // create/edit MenuItem

local aSaveFkey:=f_Fkeys("A")           // save and clear fkeys
local oP:=PopUp()
local oM:=MenuItem("New Item", oP)
local oMB:=MenuItem("New Item", {||.T.})
local aSet:=f_Set("A")
local cCaption, cCargo, cChecked, cData, cEnabled, nId, cMessage, nShortcut, cStyle
local nTyp:=1

SetColor(cColorDat)

oM:cargo:=Aclone(aNulCargo)
oMB:cargo:=Aclone(aNulCargo)
oP:AddItem(oMB)
oP:cargo:=Aclone(aNulCargo)
oP:colorSpec:=cColorSet

if oMI <> NIL
  oM:=oMI
  Do case
    case oM:isPopUp()
      nTyp:=2
    case oM:cargo[SEPARATOR]
      nTyp:=3
  endcase
endif
cCaption:=      PadR(oM:caption, 78)
cMessage:=      PadR(oM:message, 78)
cStyle  :=      oM:style
cChecked:=      PadR(oM:cargo[CHECKED], 78)
cEnabled:=      PadR(oM:cargo[ENABLED], 78)
cData   :=      PadR(oM:cargo[DATA   ], 78)
nShortCut:=     oM:cargo[SHORTCUT]

CLS
@ 0, 0 to 0, MaxCol()
@ MaxRow()-2, 0 to MaxRow()-2, MaxCol()
@ 0,30 say "< Edit MenuItem >"
if oMI==NIL                     // new empty MenuItem
  @ 1, 0 to 5, MaxCol()
  @ 3,       3 prompt "Basic menu item   " ;
        message "Creating new Basic menu item - menu item with action block"
  @ 3, col()+3 prompt "PopUp menu item   " ;
        message "creating PopUp menu item - knote for subordinate cascading menu"
  @ 3, col()+3 prompt "Separator of items" ;
        message "creating MENU_SEPARATOR - dummy item for visual separating the items"
  menu to nTyp
  DevPos(6, 0)
endif
if nTyp > 0
 SetColor(cColorDat)
 if (nTyp=1).or.(nTyp=2)
  @ row()+2, 0 say "CAPTION for MenuItem:"
  @ row()+1, 0 get cCaption
 endif
 if nTyp=1
  @ row()+1, 0 say "Action block (basic MenuItem only):"
  @ row()+1, 0 get cData when nTyp=1
  @ row()+1, 0 say "SHORTCUT key for MenuItem:"
  @ row()+1, 0 get nSHortCut picture "99999"
 endif
 if (nTyp=1).or.(nTyp=2)
  @ row()+1, 0 say "MESSAGE for MenuItem:"
  @ row()+1, 0 get cMessage
  @ row()+1, 0 say "lExpression for CHECKED:"
  @ row()+1, 0 get cChecked
  @ row()+1, 0 say "lExpression for ENABLED:"
  @ row()+1, 0 get cEnabled
  @ row()+1, 0 say "STYLE characters for CHECKED and SUBMENU:"
  @ row()+1, 0 get cStyle
 endif
 if f_Read()
  oM:caption            :=Alltrim(cCaption)
  oM:cargo[DATA]        :=Alltrim(cData)
  oM:cargo[CHECKED]     :=Alltrim(cChecked)
  oM:cargo[ENABLED]     :=Alltrim(cEnabled)
  oM:cargo[SHORTCUT]    :=nShortCut
  oM:message            :=Rtrim(cMessage)
  oM:checked            :=.not.Empty(Alltrim(cChecked))
  Do case
    case nTyp=1                 // basic item
      if oMI == NIL
        oM:data:={||.T.}
      endif
    case nTyp=2                 // popup

    case nTyp=3                 // separator
      if oMI == NIL
        oM:data:={||.T.}
      endif
      oM:caption:="-"
      oM:cargo[SEPARATOR]:=.T.
  endcase
 else
  oM:=NIL
 endif
endif
f_Fkeys("R", aSaveFkey)
f_Set("R", aSet)
return oM

/* ----------------------------------------------------------- */
static FUNCTION RedrawScr(oTB)
local oIT:=oTB:GetItem(oTB:current)     // current TopBarItem (allways present)
local oP:=NIL
local oIP

CLS
ShowHelp()
oTB:display()

if oIT:IsPopUp()                        // is current TopBarItem of PopUp type?
  oP:=oIT:data
  Do while oP:current <> 0
    oP:display()
    oIP:=oP:GetItem(oP:current)             // selected Item of PopUp
    if oIP:IsPopUp()
      oP:=oIP:data
    else
      exit
    endif
  enddo
endif
return .T.


/* ----------------------------------------------------------- */

static FUNCTION ChangeItem(oTB)         // edit current item
local oP:=GetPopUp(@oTB)                // get active PopUp
local oI
oP:=iif(oP==NIL, oTB, oP)
oI:=EditItem(oP:GetItem(oP:current))
if oI<>NIL

  oP:SetItem(oP:current, oI)
  oP:display()
endif
return .T.
/* ----------------------------------------------------------- */

static PROCEDURE GenPrg(oTB)            // generate source
local nHandle, cMemo
local cFile:="NEW_MENU.PRG"+space(44)
local aSaveFkey:=f_Fkeys("A")           // save and clear fkeys
local aSet:=f_Set("A")

SetColor(cColorDat)
nID:=0                                          // global nID for all levels of MENU

@ 10, 10 clear to 14, 69
@ 10, 10       to 14, 69
@ 10, 25 say "< Name of generated source: >"
@ 12, 12 get cFile
if f_Read()
  cFile:=Alltrim(cFile)
  nHandle:=FCreate(cFile)
  if nHandle > 0
    GenPrgH(oTB, nHandle)               // generate header
    GenLevel(oTB, nHandle, "oTopBar", 0)        // generate source body
    GenPrgF(oTB, nHandle)               // generate footer
    FClose(nHandle)
  else
    Alert("GENMENU: cannot create a file to generate the source;"+cFile)
  endif
endif
CLS
SetCursor(SC_NORMAL)
@ 0, 0 to MaxRow(), MaxCol()
@ MaxRow(), 25 say " <Ctrl><W>: Save&Exit  <Esc>: abort "
cMemo:=MemoRead(cFile)
MemoWrit(cFile, MemoEdit(cMemo, 1, 1, MaxRow()-1, MaxCol()-1, .T.) )
f_Fkeys("R", aSaveFkey)
f_Set("R", aSet)
return .T.

static PROCEDURE GenPrgH(oTB, nH)       // generate HEADING
local cR, nI
local nLev:=GetMaxLevel(oTB)

FWrite(nH, "FUNCTION CreateMenu()"+CRLF+CRLF )
For nI:=1 to nLev
  FWrite(nH, "local oP"+Alltrim(str(nI))+CRLF)
next
FWrite(nH, "local oItem"+CRLF)
cR:="local oTopBar:=TopBar("
cR:=cR+Alltrim(str(oTB:row))+", "
cR:=cR+Alltrim(str(oTB:left))+", "
cR:=cR+Alltrim(str(oTB:right))+")"+CRLF
FWrite(nH, cR)
cR:=" oTopBar:colorSpec:='"+Alltrim(oTB:colorSpec)+"'"+CRLF+CRLF
FWrite(nH, cR)
return .T.


static PROCEDURE GenPrgF(oTB, nH)               // generate FOOTING
FWrite(nH, "return oTopBar"+CRLF )
return .T.



static FUNCTION GenLevel(oP, nH, cPopUp, nLevel) // gen. BODY of SOURCE
local nI, oI, cR, oPP
local nL:=nLevel+1
local cPP:="oP"+Ltrim(str(nL))

For nI:=1 to oP:itemCount
  oI:=oP:GetItem(nI)
  oPP:=oI:data
  Do case
    case oI:isPopUp()                   // knot PopUp
      FWrite(nH, cPP+":=PopUp()"+CRLF )             // PopUp of current level
      FWrite(nH, " "+cPP+":colorSpec:='"+oPP:colorSpec+"'"+CRLF )
      cR:="oItem:=MenuItem('"
      cR:=cR+oI:caption+"', "
      cR:=cR+cPP+", , '"
      cR:=cR+Ltrim(oI:message)+"' )"+CRLF
      FWrite(nH, cR)
      FWrite(nH, cPopUp+":AddItem(oItem)"+CRLF+CRLF )
      GenLevel(oPP, nH, cPP, nL)

    case oI:cargo[SEPARATOR]            // MENU_SEPARATOR
      FWrite(nH, cPopUp+":AddItem(MENU_SEPARATOR)"+CRLF )

    otherwise                           // basic action item
      nID:=nID+1                        // global nID for menu
      cR:="oItem:=MenuItem('"
      cR:=cR+oI:caption+"', "
      cR:=cR+iif(Empty(oI:cargo[DATA]), "{||DUMMY()}", oI:cargo[DATA])+", "
      cR:=cR+Ltrim(Transform(oI:cargo[SHORTCUT], "@Z"))+", '"
      cR:=cR+Ltrim(oI:message)+"', "
      cR:=cR+Ltrim(str(nID))+" )"+CRLF
      FWrite(nH, cR)
      if .not.Empty(oI:cargo[CHECKED])
        FWrite(nH, " oItem:checked:="+Alltrim(oI:cargo[CHECKED])+CRLF)
      endif
      if .not.Empty(oI:cargo[ENABLED])
        FWrite(nH, " oItem:enabled:="+Alltrim(oI:cargo[ENABLED])+CRLF)
      endif
      if oI:style <> ""
        FWrite(nH, " oItem:style:="+oI:style+CRLF)
      endif
      FWrite(nH, cPopUp+":AddItem(oItem)"+CRLF+CRLF )
  endcase

next
return .T.


/* ----------------------------------------------------------- */

static FUNCTION GetMaxLevel(oTB)        // returns max.level of menu
local nL:=0
local oI
nLevel:=0
For nI:=1 to oTB:itemCount
  oI:=oTB:GetItem(nI)
  if oI:isPopUp()
    nLevel:=0
    GoNextLevel(oI:data)
    nL:=Max(nLevel, nL)
  endif
next
return nL

static FUNCTION GoNextLevel(oP)
local nI, oI
nLevel++
For nI:=1 to oP:itemCount
  oI:=oP:GetItem(nI)
  if oI:isPopUp()
    GoNextLevel(oI:data)
  endif
Next
return .T.


/* -----------------------------------------------------------


        GENERIC FUNCTIONS FROM JB_LIB


-------------------------------------------------------------- */

static FUNCTION f_barva(cColor, nLeftRow, nLeftCol, lExt)

/*
        cColor  pvodn nastaven barvy - pouito jako vchoz pozice v matici
        nLeftRow        souadnice umstn matice barev
        nLeftCol
        lExt            rozen matice - setblink(.F.) - * = bright background
*/
local aSet:=f_Set("A")
local aFore:={"N","B","BG","G","R","GR","RB","W","N+","B+","BG+","G+","R+","GR+","RB+","W+"}
local aBack:={"N","B","BG","G","R","GR","RB","W","N*","B*","BG*","G*","R*","GR*","RB*","W*"}
local nRow, nCol
local nRowMax:=8
local cChar:=""
local cLArrow:=chr(16)
local cRArrow:=chr(17)
local cScr
local aSplit:={}

nLeftRow:=iif(nLeftRow=NIL, 0, nLeftRow)
nLeftCol:=iif(nLeftCol=NIL, 0, nLeftCol)
nLeftRow:=Min(MaxRow()-18, nLeftRow)
nLeftCol:=Min(MaxCol()-66, nLeftCol)
lExt:=iif(lExt=NIL, .F., lExt)
cColor:=iif(Empty(cColor), "N/N", cColor)
if lExt
  nRowMax:=16
  setblink(.F.)
endif

set SCOREBOARD OFF
setcursor(SC_NONE)
setcolor("W+/N")
/*
        Zobrazen matice barev
*/
@ nLeftRow, nLeftCol clear to nLeftRow+nRowMax+1, nLeftCol+(16*4)+2
@ nLeftRow, nLeftCol       to nLeftRow+nRowMax+1, nLeftCol+(16*4)+2 DOUBLE
For nCol:=1 to 16
  for nRow:=1 to nRowMax
    @ nLeftRow+nRow, nLeftCol+(nCol*4)-2 SAY " "+cChar+" "      ;
                                         color aFore[nCol]+"/"+aBack[nRow]
  next
Next
/*
        Zjitn vchoz pozice kurzoru - dle cColor
*/
aSplit:=f_Split(cColor, "/")    // aSplit[1,1]=foreground, [2,1]=background
if Empty(aSplit)
  nCol:=1
  nRow:=1
else
  nCol:=Max(Ascan(aFore, aSplit[1,1]), 1)
  nRow:=Max(Ascan(aBack, aSplit[2,1]), 1)
  nCol:=Min(nCol, 16)
  nRow:=Min(nRow, nRowMax)
endif

Do while .T.
  cScr:=SaveScreen(nLeftRow+1, nLeftCol+1, nLeftRow+nRowMax, nLeftCol+(16*4)+1)
  @ nLeftRow+nRow, nLeftCol+(nCol*4)-3 SAY cLArrow color "W+/N"
  @ nLeftRow+nRow, nLeftCol+(nCol*4)+1 SAY cRArrow color "W+/N"
  nKey:=Inkey(0)
  Do case
    case nKey = K_ESC
      exit
    case nKey = K_ENTER
      exit
    case nKey = K_LEFT
      nCol:=iif(nCol <= 1, 16, nCol-1)
    case nKey = K_RIGHT
      nCol:=iif(nCol >= 16, 1, nCol+1)
    case nKey = K_DOWN
      nRow:=iif(nRow >=  nRowMax, 1, nRow+1)
    case nKey = K_UP
      nRow:=iif(nRow <= 1,  nRowMax, nRow-1)
  endcase
  RestScreen(nLeftRow+1, nLeftCol+1, nLeftRow+nRowMax, nLeftCol+(16*4)+1, cScr)
EndDo
f_Set("R", aSet)
return iif(nKey=K_ESC, cColor, aFore[nCol]+"/"+aBack[nRow] )


/* ----------------------------------------------------------- */
static FUNCTION f_Fkeys(cMode, aSave)   // Save, Clear, saveAndclear, Restore funkeys
#define MINFKEY -50
#define MAXFKEY 1010

local aS:={}
local nI, bK

If cMode==NIL
  return NIL
endif
cMode:=Upper(left(cMode,1))

If cMode $ "SCA"                                // save, saveAndclear, clear
  if cMode $ "SA"                       // save
    For nI:=MINFKEY  to MAXFKEY
      bK:=SetKey(nI)
      if bK <> NIL
        Aadd(aS, {nI, bK})
      endif
    next
  endif
  if cMode $ "CA"                       // clear
    For nI:=MINFKEY  to MAXFKEY
      SetKey(nI, NIL)
    next
  endif
  return aS
endif

if cMode $ "R"                                  // restore from aSave
  if aSave == NIL
    return NIL
  endif
  For nI:=1 to len(aSave)
    setkey(aSave[nI, 1], aSave[nI, 2])
  next
  return .T.
endif
return NIL
/* ----------------------------------------------------------- */



static FUNCTION f_Set(p_MODE, p_ARRAY)          // klid/nastaven SET
*       p_MODE  S       save all SETs, select  a pozici kurzoru
*                       filter, order
*               A          dtto                                 vetn obrazovky
*               R       obnov SETs a obrazovku, je-li uloena


*       p_ARRAY pro p_MODE S, A nem vznam, nemus se uvdt
*               pro p_MODE R mus bt jmnem pole, vytvoenho mdy S, A
LOCAL f_SET_A[_SET_COUNT+6], f_SET_I
local f_X

p_MODE:=Upper(left(p_MODE,1))
Do case
  case p_MODE $ "SA"
    for f_SET_I:=1 to _SET_COUNT
      f_SET_A[f_SET_I]:=set(f_SET_I)
    next
    f_SET_A[_SET_COUNT+1]:=row()
    f_SET_A[_SET_COUNT+2]:=col()
    f_SET_A[_SET_COUNT+3]:=NIL
    f_SET_A[_SET_COUNT+4]:=NIL
    f_SET_A[_SET_COUNT+5]:=NIL
    f_SET_A[_SET_COUNT+6]:=NIL
    if p_MODE=="A"
      save screen to f_SET_A[_SET_COUNT+3]
    endif
    if Used()                                   // inf. o default workarea
      f_SET_A[_SET_COUNT+4]:=str(select())
      f_SET_A[_SET_COUNT+5]:=DBfilter()
      f_SET_A[_SET_COUNT+6]:=str(IndexOrd())
    endif
  case p_MODE $ "FR"
    for f_SET_I:=1 to _SET_COUNT
      set(f_SET_I, p_ARRAY[f_SET_I])
    next
    SetPos(p_ARRAY[_SET_COUNT+1], p_ARRAY[_SET_COUNT+2])
    if p_ARRAY[_SET_COUNT+3]<>NIL
      restore screen from p_ARRAY[_SET_COUNT+3]
    endif
    if p_ARRAY[_SET_COUNT+4] <> NIL             // inf. o default workarea
      select(p_ARRAY[_SET_COUNT+4])
      f_X:=p_ARRAY[_SET_COUNT+5]
      set filter to &f_X.
      f_X:=p_ARRAY[_SET_COUNT+6]
      set order to (f_X)
    endif
  EndCase
return f_SET_A

/* ----------------------------------------------------------- */
static FUNCTION  f_SPLIT(p_T, p_O)
*       Vrac pole, do nho rozhod poloky etzce p_T, oddlen oddlovaem p_O.
*       Oddlovaem p_O je libovoln etzec - seznam znak, povaovanch jednotliv
*       za oddlovae.
*       Poloka pole[x, 1] obsahuje sloku p_T, poloka pole[x, 2]
*       pole oddlova p_O mezi danou slokou a slokou nsledujc.

local aSplit:={}
local cPolozka, cOddelovac
local x_CITAC:=0

if p_T = NIL
  return {}
endif
*       1. zjitn potu sloek
x_T:=p_T
Do while len(p_T) > 0
  x_CITAC++
  x_I:=f_At(p_O, p_T)                            && zan oddlovaem?
  x_I:=iif(x_I=0, len(p_T)+1, x_I)
  if x_I <> 1
    p_T:=right(p_T, len(p_T)-x_I+1)                && odmazn sloky z p_T
  endif
  do while len(p_T) > 0                         && odmazn oddlova
    if .not.(left(p_T,1) $ p_O)
      exit
    endif
    p_T:=right(p_T, len(p_T)-1)                  && odmazn vodnho oddlovae
  enddo
EndDo
*       2. rozhozen do pole aSplit
x_CITAC:=0
Do while len(x_T) > 0
  x_CITAC++
  x_I=f_At(p_O, x_T)                            && zan oddlovaem?
  x_I=iif(x_I=0, len(x_T)+1, x_I)
  cPolozka:=""
  if x_I <> 1
    cPolozka:=left(x_T, x_I-1)          && sloka do pole
    x_T:=right(x_T, len(x_T)-x_I+1)                && odmazn sloky z x_T
  endif
  cOddelovac:=""
  do while len(x_T) > 0                         && odmazn oddlova
    if .not.(left(x_T,1) $ p_O)
      exit
    endif
    cOddelovac:=cOddelovac+left(x_T,1)
    x_T=right(x_T, len(x_T)-1)                  && odmazn vodnho oddlovae
  enddo
  Aadd(aSplit, {cPolozka, cOddelovac} )
EndDo
return aSplit

/* ----------------------------------------------------------- */
static FUNCTION f_AT(p_V, p_T)
*       funkce f_at hled prvn vskyt kterhokoliv znaku
*       z etzce p_V v etzci p_T. Vrac jeho pozici;
*       nebyl-li nalezen, pak 0.
x_POS:=len(p_T)+1
Do while len(p_V) > 0
  x_P:=at(left(p_V,1), p_T)
  x_POS:=iif((x_P>0).and.(x_P<x_POS), x_P, x_POS)
  p_V:=right(p_V, len(p_V)-1)
EndDo
x_POS:=iif(x_POS=len(p_T)+1, 0, x_POS)
return x_POS


/* ----------------------------------------------------------- */

static FUNCTION f_read()
*
*       Provede prikaz READ - precteni pripravenych GET
*       Vraci:  .T.     nebylo Esc
*               .F.     bylo stisknuto Esc
*
local f_Read_:=f_Set("S")
local f_ReadIO:=ReadInsert(.F.)
set(_SET_CONSOLE, .T.)
set(_SET_DEVICE, "SCREEN")
SetCursor(SC_NORMAL)
read
ReadInsert(f_ReadIO)
f_SET("R", f_Read_)
return LastKey() <> K_ESC


/* ----------------------------------------------------------- */
